@@ -6,6 +6,7 @@ import os
 import logging
 import time
 import tempfile
+from datetime import datetime
 from dataclasses import dataclass
 from contextlib import contextmanager
 from uuid import uuid1
@@ -22,8 +23,9 @@ class Allocation:
     """
     requirements: dict
     resource_info: dict
-    release: callable
+    _release: callable
     pid_file: str
+    allocation_time: datetime = datetime.now()
     alloc_id: str = str(uuid1())
 
     @property
@@ -31,6 +33,12 @@ class Allocation:
         """ resource id getter """
         return self.resource_info['id']
 
+    def release(self, alloc_id):
+        """ Release resource """
+        assert self.alloc_id == alloc_id, 'Allocation id mismatch'
+        self._release()
+        self.alloc_id = None
+
 
 class ResourceNotFound(Exception):
     """ Exception raised when resource not found """
@@ -48,25 +56,37 @@ class Lockable:
     """
     def __init__(self, hostname=socket.gethostname(),
                  resource_list_file=None,
+                 resource_list=None,
                  lock_folder=tempfile.gettempdir()):
-        self._resources = dict()
+        self._allocations = dict()
         self.logger = logging.getLogger('lockable.Lockable')
         self.logger.debug('Initialized lockable')
         self._hostname = hostname
         self._lock_folder = lock_folder
         self._resource_list = None
-        if resource_list_file:
-            self.load_resources_list(resource_list_file)
+        assert not (isinstance(resource_list, list) and
+                    resource_list_file), 'only one of resource_list or ' \
+                                         'resource_list_file is accepted, not both'
+        if isinstance(resource_list_file, str):
+            self.load_resources_list_file(resource_list_file)
+        elif isinstance(resource_list, list):
+            self.load_resources_list(resource_list)
         else:
-            self.logger.warning('resource_list_file is not configured')
+            self.logger.warning('resource_list_file or resource_list is not configured')
 
-    def load_resources_list(self, filename: str):
+    def load_resources_list_file(self, filename: str):
+        """ Load resources list file"""
+        self.load_resources_list(self._read_resources_list(filename))
+        self.logger.warning('Use resources from %s file', filename)
+
+    def load_resources_list(self, resources_list: list):
         """ Load resources list """
-        self._resource_list = self._read_resources_list(filename)
-        self.logger.debug('Resources: ')
+        assert isinstance(resources_list, list), 'resources_list is not an list'
+        self._resource_list = resources_list
+        self.logger.debug('Resources loaded: ')
         for resource in self._resource_list:
             self.logger.debug(json.dumps(resource))
-        self.logger.warning('Use resources from %s file', filename)
+
 
     @staticmethod
     def _read_resources_list(filename):
@@ -95,7 +115,7 @@ class Lockable:
             raise ValueError(f"Invalid json, duplicate ids in {duplicates}")
 
     @staticmethod
-    def parse_requirements(requirements_str: (str or dict)):
+    def parse_requirements(requirements_str: (str or dict)) -> dict:
         """ Parse requirements """
         if not requirements_str:
             return dict()
@@ -127,10 +147,10 @@ class Lockable:
         """ Function that tries to lock given candidate resource """
         resource_id = candidate.get("id")
         try:
-            pid_file = os.path.join(self._lock_folder, f"{resource_id}.pid")
-            self.logger.debug('Trying lock using: %s', pid_file)
+            pid_file = f"{resource_id}.pid"
+            self.logger.debug('Trying lock using: %s', os.path.join(self._lock_folder, pid_file))
 
-            _lockable = PidFile(pidname=pid_file)
+            _lockable = PidFile(pidname=pid_file, piddir=self._lock_folder)
             _lockable.create()
             self.logger.info('Allocated: %s, lockfile: %s', resource_id, pid_file)
 
@@ -138,12 +158,12 @@ class Lockable:
                 nonlocal self, resource_id, _lockable
                 self.logger.info('Release resource: %s', resource_id)
                 _lockable.close()
-                del self._resources[resource_id]
+                del self._allocations[resource_id]
 
             return Allocation(requirements=requirements,
                               resource_info=candidate,
-                              release=release,
-                              pid_file=pid_file)
+                              _release=release,
+                              pid_file=_lockable.filename)
         except PidFileError as error:
             raise AssertionError('no success') from error
 
@@ -158,8 +178,10 @@ class Lockable:
             for candidate in candidates:
                 try:
                     allocation = self._try_lock(requirements, candidate)
-                    self.logger.debug('resource %s allocated (%s)',
-                                      allocation.resource_id, json.dumps(allocation.resource_info))
+                    self.logger.debug('resource %s allocated (%s), alloc_id: (%s)',
+                                      allocation.resource_id,
+                                      json.dumps(allocation.resource_info),
+                                      allocation.alloc_id)
                     return allocation
                 except AssertionError:
                     pass
@@ -186,12 +208,12 @@ class Lockable:
         MODULE_LOGGER.debug('hostname: %s', hostname)
         return merge(dict(hostname=hostname, online=True), requirements)
 
-    def lock(self, requirements: (str or dict), timeout_s: int = 1000):
+    def lock(self, requirements: (str or dict), timeout_s: int = 1000) -> Allocation:
         """
         Lock resource
         :param requirements: resource requirements
         :param timeout_s: timeout while trying to lock
-        :return:
+        :return: Allocation context
         """
         assert isinstance(self._resource_list, list), 'resources list is not loaded'
         requirements = self.parse_requirements(requirements)
@@ -199,33 +221,33 @@ class Lockable:
         self.logger.debug("Use lock folder: %s", self._lock_folder)
         self.logger.debug("Requirements: %s", json.dumps(predicate))
         self.logger.debug("Resource list: %s", json.dumps(self._resource_list))
-        reservation = self._lock(predicate, timeout_s)
-        self._resources[reservation.resource_id] = reservation
-        return reservation
+        allocation = self._lock(predicate, timeout_s)
+        self._allocations[allocation.resource_id] = allocation
+        return allocation
 
-    def unlock(self, resource: dict):
+    def unlock(self, allocation: Allocation) -> None:
         """
         Method to release resource
-        :param resource: resource object. 'id' property required.
+        :param allocation: Allocation object.
         :return: None
         """
-        assert 'id' in resource, 'missing "id" -key'
-        self.logger.info('Release: %s', resource)
-        resource_id = resource['id']
-        ResourceNotFound.invariant(resource_id in self._resources.keys(), 'resource not locked')
-        reservation = self._resources[resource_id]
-        reservation.release()
+        assert 'id' in allocation.resource_info, 'missing "id" -key'
+        self.logger.info('Release: %s', allocation.resource_id)
+        resource_id = allocation.resource_id
+        ResourceNotFound.invariant(resource_id in self._allocations.keys(), 'resource not locked')
+        reservation = self._allocations[resource_id]
+        reservation.release(allocation.alloc_id)
 
     @contextmanager
-    def auto_lock(self, requirements: (str or dict), timeout_s: int = 0):
+    def auto_lock(self, requirements: (str or dict), timeout_s: int = 0) -> Allocation:
         """
         contextmanaged lock method. Resource is released automatically after context ends.
         :param requirements: requirements
         :param timeout_s: timeout while trying to lock suitable resource
-        :return: return resource info object
+        :return: return Allocation object
         """
         allocator = self.lock(requirements=requirements, timeout_s=timeout_s)
         try:
-            yield allocator.resource_info
+            yield allocator
         finally:
-            allocator.release()
+            allocator.release(allocator.alloc_id)
