@@ -1,9 +1,18 @@
 import os
 import logging
 import json
+import dataclasses
+from contextlib import contextmanager
 from tempfile import TemporaryDirectory
 from unittest import TestCase
-from lockable.lockable import Lockable, ResourceNotFound
+from lockable.lockable import Lockable, ResourceNotFound, Allocation
+
+
+@contextmanager
+def create_lockable(data=[{"id": 1, "hostname": "myhost", "online": True}], lock_folder=None):
+    with TemporaryDirectory() as tmpdirname:
+        lock_folder = lock_folder or tmpdirname
+        yield Lockable(hostname='myhost', resource_list=data, lock_folder=lock_folder)
 
 
 class LockableTests(TestCase):
@@ -20,6 +29,11 @@ class LockableTests(TestCase):
                 fp.write('[]')
             Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
 
+    def test_invalid_constructor(self):
+        with self.assertRaises(AssertionError):
+            Lockable(hostname='myhost', resource_list_file='asdf',
+                     resource_list=[], lock_folder='.')
+
     def test_lock_require_resources_json_loaded(self):
         lockable = Lockable()
         with self.assertRaises(AssertionError) as error:
@@ -73,71 +87,61 @@ class LockableTests(TestCase):
             Lockable.parse_requirements('{"a":"b","c":"d}')
 
     def test_lock_resource_not_found(self):
-        with TemporaryDirectory() as tmpdirname:
-            list_file = os.path.join(tmpdirname, 'test.json')
-            with open(list_file, 'w') as fp:
-                fp.write('[]')
-            lockable = Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
+        with create_lockable([]) as lockable:
             with self.assertRaises(ResourceNotFound):
                 lockable.lock({})
 
     def test_lock_timeout_0(self):
-        with TemporaryDirectory() as tmpdirname:
-            list_file = os.path.join(tmpdirname, 'test.json')
-            with open(list_file, 'w') as fp:
-                fp.write('[{"id": 1, "hostname": "myhost", "online": true}]')
-            lock_file = os.path.join(tmpdirname, "1.pid")
+        with create_lockable([{"id": 1, "hostname": "myhost", "online": True}]) as lockable:
+            lock_file = os.path.join(lockable._lock_folder, "1.pid")
             with open(lock_file, 'w') as fp:
                 fp.write(f'{os.getpid()}')
-            lockable = Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
             with self.assertRaises(TimeoutError):
                 lockable.lock({}, timeout_s=0)
             os.unlink(lock_file)
 
     def test_lock_timeout_0_success(self):
-        with TemporaryDirectory() as tmpdirname:
-            list_file = os.path.join(tmpdirname, 'test.json')
-            with open(list_file, 'w') as fp:
-                fp.write('[{"id": 1, "hostname": "myhost", "online": true}]')
-            lock_file = os.path.join(tmpdirname, '1.pid')
-            lockable = Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
+        with create_lockable([{"id": 1, "hostname": "myhost", "online": True}],
+                             lock_folder='.') as lockable:
+            lock_file = os.path.join(".", "1.pid")
             object = lockable.lock({}, timeout_s=0)
             self.assertTrue(os.path.exists(lock_file))
-            object.release()
+            object.release(object.alloc_id)
             self.assertFalse(os.path.exists(lock_file))
 
     def test_lock_timeout_1(self):
-        with TemporaryDirectory() as tmpdirname:
-            list_file = os.path.join(tmpdirname, 'test.json')
-            with open(list_file, 'w') as fp:
-                fp.write('[{"id": 1, "hostname": "myhost", "online": true}]')
-            lock_file = os.path.join(tmpdirname, "1.pid")
+        with create_lockable([{"id": 1, "hostname": "myhost", "online": True}]) as lockable:
+            lock_file = os.path.join(lockable._lock_folder, "1.pid")
             with open(lock_file, 'w') as fp:
                 fp.write(f'{os.getpid()}')
-            lockable = Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
             with self.assertRaises(TimeoutError):
                 lockable.lock({}, timeout_s=1)
             os.unlink(lock_file)
 
     def test_unlock(self):
-        with TemporaryDirectory() as tmpdirname:
-            list_file = os.path.join(tmpdirname, 'test.json')
-            with open(list_file, 'w') as fp:
-                fp.write('[{"id": 1, "hostname": "myhost", "online": true}]')
-            lock_file = os.path.join(tmpdirname, '1.pid')
-            lockable = Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
-
-            with self.assertRaises(AssertionError):
-                lockable.unlock({'a': 2})
-
-            with self.assertRaises(ResourceNotFound):
-                lockable.unlock({'id': 2})
+        with create_lockable([{"id": 1, "hostname": "myhost", "online": True}]) as lockable:
+            lock_file = os.path.join(lockable._lock_folder, "1.pid")
 
             allocation = lockable.lock({}, timeout_s=0)
             self.assertTrue(os.path.exists(lock_file))
-            lockable.unlock(allocation.resource_info)
+
+            with self.assertRaises(AssertionError):
+                alloc = dataclasses.replace(allocation)
+                alloc.alloc_id = '123'
+                lockable.unlock(alloc)
+
+            with self.assertRaises(ResourceNotFound):
+                alloc = dataclasses.replace(allocation)
+                alloc.resource_info = alloc.resource_info.copy()
+                alloc.resource_info['id'] = '2'
+                lockable.unlock(alloc)
+
+            lockable.unlock(allocation)
             self.assertFalse(os.path.exists(lock_file))
 
+
+    #def test_unlock_fails_with_invalid_alloc_id(self):
+
     def test_lock_offline(self):
         with TemporaryDirectory() as tmpdirname:
             list_file = os.path.join(tmpdirname, 'test.json')
@@ -160,7 +164,8 @@ class LockableTests(TestCase):
             lockable = Lockable(hostname='myhost', resource_list_file=list_file, lock_folder=tmpdirname)
             lock_file = os.path.join(tmpdirname, '1.pid')
             self.assertFalse(os.path.exists(lock_file))
-            with lockable.auto_lock({}) as resource:
+            with lockable.auto_lock({}) as context:
+                resource = context.resource_info
                 self.assertEqual(resource, resource_info)
                 self.assertTrue(os.path.exists(lock_file))
             self.assertFalse(os.path.exists(lock_file))
